// Sudokus con Clase
// Resolución de sudokus con técnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Declaración de la clase Columna

#ifndef CAJA_H
#define CAJA_H

#include "grupo.h"

class Caja : public Grupo
{
    private:
    int punteroFila[3][9];
    int punteroColumna[3][9];

    public:
    Caja();                                                         /**< Constructor */
    virtual void Iniciar(Tablero &t, int n);                        /**< Iniciar caja */
    void IniciaPunteros();                                          /**< Inicializa los punteros de filas y columnas */
    int CuentaPosiblesFila(int f);                                  /**< Cuenta posibles en fila f de la caja */
    int CuentaPosiblesColumna(int c);                               /**< Cuenta posibles en columna c de la caja */
    bool AsignaPunteroFila(int f, int v, int n);                    /**< Almacena un par o trio puntero (n) de valor v a la fila c */
    bool AsignaPunteroColumna(int c, int v, int n);                 /**< Almacena un par o trio puntero (n) de valor v a la columna c */
    bool PunteroFila(int f, int v) const                            /**< Devuelve 2 ó 3 si existe un puntero en la fila f para el valor v */
        { return punteroFila[f][v-1]; }
    bool PunteroColumna(int c, int v) const                         /**< Devuelve 2 ó 3 si existe un puntero en la columna c para el valor v */
        { return punteroColumna[c][v-1]; }
    Casilla *BuscaAla2MenosFila(Casilla *pivote, Casilla *ala1);
    Casilla *BuscaAla2MenosColumna(Casilla *pivote, Casilla *ala1);
    // Rectángulos vacíos
    int RectanguloVacio(int v);                                     /**< Busca un patrón "rectángulo vacío" */
    // Hilo de cometa
    Pareja DobleCandidato(int v);                                   /**< Busca una pareja de casiilas con las misma pareja de candidatos */
    virtual Casilla *DobleCandidatoYMas(Pareja par, int v) {return nullptr;}
};

#endif // CAJA_H
