// Sudokus con Clase
// Resolución de sudokus con técnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Declaración de la clase Casilla

#ifndef CASILLA_H
#define CASILLA_H

#include "marcaslapiz.h"

class Casilla
{
    private:
    int fila,columna,caja,pos;  /**< Posición de la casilla en el tablero: fila, columna, y posición en caja */
    int valor;                  /**< Valor final, o 0 si no tiene valor */
    bool posible;               /**< Indica si un candidato es posible en cribas, color verde */
    bool inicial;               /**< Indica si se trata de una pista del juego, un valor incial */
    bool resaltar;              /**< Resaltar la casilla color azul */
    bool amarillo;              /**< Mostrar en color amarillo */
    bool naranja;               /**< Mostrar en color naranja */
    MarcasLapiz marcas;         /**< Marcas de lápiz de la casilla */
    char nombre[3];             /**< Nombre de la casilla, por ejemplo A1 o C5 */
    Casilla* pardesnudo[3];     /**< Indica si la casilla forma parte de un par desnudo */

    public:
    enum tipopardesnudo {
        enfila=0,
        encolumna,
        encaja
    };
    Casilla();                                                   /**< Constructor por defecto */
    Casilla(int f, int c, int v, bool ini);                     /**< Constructor */
    const char* Nombre() const { return nombre; }               /**< Devuelve el nombre de la casilla */
    int Valor() const { return valor; }                         /**< Obtiene el valor */
    bool Inicial() { return inicial; }                          /**< Devuelve true si es una casilla de pista */
    void Asignar(int v);                                        /**< Asigna un valor */
    int Fila() const { return fila; }                           /**< Obtiene la fila */
    int Columna() const { return columna; }                     /**< Obtiene la columna */
    int Caja() const { return caja; }                           /**< Obtiene la caja */
    int PosEnCaja() const { return pos; }                       /**< Obtiene la posición dentro de la caja */
    bool Posible() const { return posible; }                    /**< Obtiene la bandera "posible" (verde) */
    bool Resaltado() const { return resaltar; }                 /**< Obtiene el valor de la bandera "resaltar" (azul) */
    bool EsAmarillo() const { return amarillo; }                /**< Obtiene el valor de la bandera "amarillo" */
    bool EsNaranja() const { return naranja; }                  /**< Obtiene el valor de la bandera "naranja" */
    void Resaltar(bool r=true) { resaltar = r; }                /**< Resalta la casilla */
    void Amarillo(bool a=true) { amarillo = a; }                /**< Activa el color amarillo */
    void Naranja(bool n=true) { naranja = n; }                  /**< Activa el color naranja */
    void MarcaPosible( bool p=true ) { posible = p; }           /**< Marca la casilla como posible (parámetro) */
    void MarcaNoPosible() { posible = false; }                  /**< Desmarca la casilla como posible */
    MarcasLapiz &Lapiz() { return marcas; }                     /**< Obtiene las marcas de lápiz */
    void MarcaParDesnudo(Casilla* cas, tipopardesnudo t);       /**< Marca la casilla como parte de un par desnudo */
    Casilla* ParDesnudo(tipopardesnudo t) const;                /**< Devuelve la pareja de esta casilla si forma parte de un par desnudo */
    bool ParDesnudo();                                          /**< Devuelve true si la casilla forma parte de un par desnudo */
    void DestruyeParDesnudo();                                  /**< Destruye cualquier par desnudo al que pertenezca esta casilla */
};

#endif // CASILLA_H
