// Sudokus con Clase
// Resolución de sudokus con técnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Declaración de la clase abstracta Grupo, de la que derivan Fila, Columna y Caja

//#include "tablero.h"

#ifndef GRUPO_H
#define GRUPO_H

#include "casilla.h"
#include "unico.h"
#include "pareja.h"
#include "marcascasillas.h"

class Tablero;

class Grupo
{
    protected:
    Casilla *cas[9];        /**< Punteros a las casillas que forman el grupo de una fila, columna o caja */

    public:
    Grupo() { for(int i=0; i < 9; i++) cas[i] = nullptr; }  /**< Constructor  */
    virtual void Iniciar(Tablero &t, int n) = 0;            /**< Inicializa el array de casillas con el grupo n */
    bool Presente(int n);                                   /**< Devuelve true si el valor n está asignado a alguna casilla del grupo */
    int CuentaPosibles();                                   /**< Cuenta casillas "posibles" */
    Casilla *Cas(int i) { return cas[i]; }                  /**< Obtiene un puntero a la casilla i del grupo */
    Casilla **GetCasillas() { return cas; }                 /**< Obtiene un puntero a todas las casillas del grupo */

    void Criba();                                           /**< Marca todas las casillas del grupo como "no posibles" */
    void CribaMenosCaja(int p);                             /**< Marca todas las casillas del grupo como "no posibles" menos las de la fila/columna de cajas p */
    Unico CandidatoUnico();                                 /**< Devuelve la única casilla sin valor final, y su valor, si existe */
    int UnicoPosible();                                     /**< Después de una criba, devuelve el índice de la única casilla con la marca "posible", o -1 */
    int CuentaFinales();                                    /**< Cuenta casillas con valor final */
    int PrimeroPosible();                                   /**< Devuelve el índice de la primera casilla "posible" */
    int SegundoPosible(int p);                              /**< Devuelve el índice de la segunda casilla "posible", o -1 */
    int TerceroPosible(int t);                              /**< Devuelve el índice de la tercera casilla "posible", o -1 */
    void DesmarcaLapiz(int v);                              /**< Elimina la marca de lápiz del valor v de todo el grupo */
    void DesmarcaLapizMenosCaja(int k, int v);              /**< Elimina la marca de lápiz del valor v de todo el grupo, menos las de la caja k */
    Unico UnicaCasillaConCandidato();                       /**< Devuelve la casilla del grupo que tiene la única marca de lápiz para un valor  */
    // XY-Wing
    Casilla *BuscaAla1MenosCaja(Casilla *pivote, int c1=0); /**< Busca la casilla que falta para un patrón XY-Wing */
    Casilla *BuscaAla2MenosCaja(Casilla *pivote, Casilla *ala1);

    // X-Wing
    Pareja DobleCandidato(int v);                           /**< Busca una pareja de casillas con la misma pareja de candidatos */
    // X-Wing-alas
    virtual Casilla *DobleCandidatoYMas(Pareja par, int v)=0;/**< Busca patrones X-Wing con alas, depende de si son filas o columnas */
    // SwordFish
    int CuentaCandidato(int v);                             /**< Devuelve el número de casillas con v como candidato */
    int BuscarMarcas(const MarcasLapiz &m);                 /**< Devuelve el índice de la casilla con las marcas de lápiz indicadas */
    MarcasCasillas Marcas(int v);                           /**< Devuelve las casillas que contienen la marca de lápiz v */
    int CasillasConCandidato(int v);                        /**< Devuelve el número de casillas en el grupos con el candidato v */
};

#endif // GRUPO_H
