// Resolución de sudokus con técnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Declaración de la clase MarcasLapiz

#ifndef MARCASLAPIZ_H
#define MARCASLAPIZ_H

class MarcasLapiz
{
    private:
    bool lapiz[9];          /**< Bits que indican la presencia de marcas de lápiz */
    bool eliminada[9];      /**< Marcas eliminadas en último método */

    public:
    MarcasLapiz()                                               /** Constructor por defecto */
        { for(int i=0; i<9; i++) lapiz[i] = false; }
    MarcasLapiz(const MarcasLapiz &m)                           /**< Constructor copia */
        { for(int i=0; i<9; i++) lapiz[i] = m.lapiz[i]; }
    bool AsignaMarca(int v);                                    /**< Establece la marca de lápiz correspondiente al valor v, retorna false si ya estaba marcada */
    void MarcarTodas()                                          /**< Activa todas las marcas */
        { for(int i=0; i<9; i++) lapiz[i] = true; }
    bool EliminaMarca(int v);                                   /**< Elimina la marca de lápiz correspondiente al valor v, retorna false si no estaba marcada */
    void ResetEliminadas();                                     /**< Elimina las marcas de candidatos eliminados */
    bool ContieneEliminada(int v) { return eliminada[v-1]; }    /**< Devuelve true si el candidato v fue eliminado */
    int CuentaMarcas();                                         /**< Cuenta el número de marcas de lápiz activas */
    bool ContieneMarca(int v) { return lapiz[v-1]; }            /**< Devuelve true si la marca de lápiz correspondiente a v está activa */
    MarcasLapiz &operator=(const MarcasLapiz &m1)               /**< Operador de asignación */
        { for(int i=0; i<9; i++) lapiz[i] = m1.lapiz[i]; return *this; };
    MarcasLapiz operator|(const MarcasLapiz &m1);               /**< Operador OR */
    MarcasLapiz operator&(const MarcasLapiz &m1);               /**< Operador AND */
    MarcasLapiz operator-(const MarcasLapiz &m1);               /**< Operador resta */
    bool operator==(const MarcasLapiz &m1);                     /**< Operador de igualdad */
    bool operator!=(const MarcasLapiz &m1);                     /**< Operador de desigualdad */
    bool Quitar(const MarcasLapiz &m1);                         /**< Eliminar las marcas de lápiz persentes en el parámetro */
    char* Valores();                                            /**< Lista de valores de marcas activas en forma de texto */
    int Valor();                                                /**< Valor si las marcas de lápiz solo contienen una marca */
    int ValorMarca(int n);                                      /**< Devuelve el valor asociado a la marca n, p.e. si marcas son 3,4,6,8, el valor de la marca 3 es 6 */
};

#endif // MARCASLAPIZ_H
