// Sudokus con Clase
// Resolución de sudokus con técnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Declaración de la clase Tablero

#ifndef TABLERO_H
#define TABLERO_H

#include <vector>
#include <cstdio>
#include <queue>
#include <string>
#include <cstring>
#include "caja.h"
#include "fila.h"
#include "columna.h"
#include "casilla.h"
#include "lista.h"
#include "oculto.h"
#include "puntero.h"
#include "pareja.h"
#include "marcascasillas.h"

using std::sprintf;
using std::strcat;
using std::strcpy;

enum eEstado {
    inicio=1,
    cribado,
    ochofinales,
    solitarios,
    paresdesnudos,
    marcaslapiz,
    unamarcalapiz,
    candidatounicofila,
    candidatounicocolumna,
    candidatounicocaja,
    buscaparesdesnudos2,
    procesaparesdesnudos2,
    punteros,
    grupos2,
    grupos3,
    grupos4,
    grupos5,
    xywing,
    xwing,
    xwingalas,
    pezespada,
    doscandidatos,
    solucionunicaA,
    solucionunicaB,
    xyzwing,
    sashimi,
    skyscraper,
    rectangulovacio,
    medusa,
    stringkite,
    pezespadaalas,
    nosolucionable=100,
    resuelto=200
};

class Tablero {
    private:
    std::queue<std::string> msgs;
    std::string tecnica;
    std::string cad;
    char m[250];
    std::vector<std::vector<Casilla>> casilla;
    Fila fila[9];
    Columna columna[9];
    Caja caja[9];
    int finales;
    int cuenta;
    int estado;
    Lista<Puntero> puntero;

    bool FuerzaBruta();
    void ResetCriba();

    bool ProcesarCriba(int v);
    bool BuscaPunterosFila(int f, int v);
    bool BuscaPunterosColumna(int c, int v);
    bool BuscaPunterosCaja(int k, int v);
    bool CribaFilasCajas(int v);
    bool CribaColumnasCajas(int v);
    void VerificarParDesnudo(Casilla* c1, Casilla* c2, int v);

    bool CandidatoUnicoFila(int f);
    bool CandidatoUnicoColumna(int c);
    bool CandidatoUnicoCaja(int k);

    bool BuscaParesDesnudosFilas();
    bool BuscaParesDesnudosColumnas();
    bool BuscaParesDesnudosCajas();

    bool ProcesarDesnudoFila(int c, bool todos=false);
    bool ProcesarDesnudoColumna(int c, bool todos=false);
    bool ProcesarDesnudoCaja(int c);

    bool ProcesaPunterosFila(int f);
    bool ProcesaPunterosColumna(int c);
    bool ProcesaPunterosCaja(int k);

    bool ProcesaGrupoFila(int grupo, int nElementos);
    bool ProcesaGrupoColumna(int grupo, int nElementos);
    bool ProcesaGrupoCaja(int grupo, int nElementos);

    bool ProcesarXYWingCaso1(Casilla *pivote, Casilla *ala1, Casilla *ala2);
    bool ProcesarXYWingCaso2(Casilla *pivote, Casilla *ala1, Casilla *ala2);
    bool ProcesarXYWingCaso3(Casilla *pivote, Casilla *ala1, Casilla *ala2);

    bool ProcesaXWingFila(Pareja a, Pareja b, int v);
    bool ProcesaXWingColumna(Pareja a, Pareja b, int v);

    bool BuscaXWingAlasFilas(int v);
    bool BuscaXWingAlasColumnas(int v);
    bool ProcesaXWingAlasFilas(Pareja par, Casilla *f2, int v);
    bool ProcesaXWingAlasColumnas(Pareja par, Casilla *c2, int v);

    bool BuscaSwordFishFilas(int v);
    bool BuscaSwordFishColumnas(int v);
    bool ProcesaSwordFishFilas(MarcasCasillas, MarcasCasillas, int v);
    bool ProcesaSwordFishColumnas(MarcasCasillas, MarcasCasillas, int v);

    bool BuscaFila(Casilla& cas);
    bool BuscaColumna(Casilla& cas);

    bool BuscaFilaTrios(Casilla& cas);
    bool BuscaColumnaTrios(Casilla& cas);

    bool BuscaFilaTriosB(Casilla& cas);
    bool BuscaColumnaTriosB(Casilla& cas);

    bool ProcesarXYZFila(int k, int fil, int col, int v, int c, int p);
    bool ProcesarXYZColumna(int k, int fil, int col, int v, int f, int p);

    bool BuscaSashimiFilas();
    bool BuscaSashimiColumnas();
    bool ProcesarSashimiFila(int f1, int f2, int c1, int c2, int v, int c, int ca, int cb);
    bool ProcesarSashimiColumna(int f1, int f2, int c1, int c2, int v, int f, int fa, int fb);

    bool BuscaSkyscraperFilas();
    bool BuscaSkyscraperColumnas();
    bool ProcesarSkyscraperFila(Pareja &par1, Pareja &par2, int np, int v);
    bool ProcesarSkyscraperColumna(Pareja &par1, Pareja &par2, int np, int v);

    bool BuscaJellyFishFilas(int v);
    bool BuscaJellyFishColumnas(int v);
    bool ProcesaJellyFishFilas(MarcasCasillas, MarcasCasillas, int v);
    bool ProcesaJellyFishColumnas(MarcasCasillas, MarcasCasillas, int v);

    bool ProcesarStringKite(Pareja &par, Pareja &parF, Pareja &parC, int v);

    bool BuscaSwordFishAlasFilas(int v);
    bool BuscaSwordFishAlasColumnas(int v);
    bool ProcesaSwordFishAlasFilas(MarcasCasillas &CasillasFilas,
                                   MarcasCasillas &CasillasColumnas,
                                   MarcasCasillas &CasillasAlas,
                                   int f3, int v);
    bool ProcesaSwordFishAlasColumnas(MarcasCasillas &CasillasFilas,
                                      MarcasCasillas &CasillasColumnas,
                                      MarcasCasillas &CasillasAlas,
                                      int c3, int v);
    public:
    Tablero();
    ~Tablero() {}
    void Leer(const unsigned char *t);

    bool Mensaje(std::string& cad) {
        if(msgs.empty()) return false;
        cad = msgs.front();
        msgs.pop();
        return true;
    }
    std::string& Tecnica() { return tecnica; }
    Casilla &GetCasilla(int f, int c) { return casilla[f][c]; }
    const int Cuenta() { return cuenta; }
    void SetCuenta(int c) { cuenta = c; }
    void Soluciones();
    bool Valido(int f, int c, int v);
    bool Asignar(int f, int c, int v);
    bool Asignar(Casilla &cas, int v);
    int Estado() const { return estado; }
    void SetEstado(int e) { estado = e; }
    bool Terminado() const { return finales==81; }
    bool VerificarTerminadoCorrecto();
    bool VerificarSolucionPosible();

    // Resolución:
    void LimpiaResaltes();
    void BorraCriba();
    bool Criba(int v);

    bool BuscaParesDesnudos(bool todos=true);
    bool ProcesarParesDesnudos(bool todos=true);

    // Resolucion FASE 2
    void PonerMarcasLapiz();
    bool CasillasConUnaMarcaLapiz();
    bool UnicaCasillaConCandidato();

    bool BuscaPunteros();
    bool ProcesaPunteros();

    bool ProcesaGrupo(int nElementos);
    bool ProcesaXYWing();
    bool ProcesaXWing();
    bool ProcesaXWingAlas();
    bool ProcesaSwordFish();
    bool TodasConDosCandidatosMenosUna();
    bool SolucionUnicaA();
    bool SolucionUnicaB();
    bool ProcesaXYZWing();
    bool ProcesaSashimi();
    bool ProcesaSkyscraper();
    bool ProcesaRectangulosVacios();
    bool ProcesaJellyFish();
    bool StringKite();
    bool SwordFishAlas();

    Lista<Puntero> *ListaPuntero() { return &puntero; }
};

#endif // TABLERO_H
