// Sudokus con Clase
// Resolución de sudokus con técnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definición de la clase Casilla

#include "casilla.h"

Casilla::Casilla() : fila(), columna(0), valor(0), inicial(false), marcas() {
    pardesnudo[enfila] = nullptr;
    pardesnudo[encolumna] = nullptr;
    pardesnudo[encaja] = nullptr;
}

Casilla::Casilla(int f, int c, int v, bool ini) :
    fila(f), columna(c), caja((f/3)*3+c/3), pos((fila%3)*3+columna%3), valor(v),
    posible(false), inicial(ini), resaltar(false), amarillo(false), naranja(false), marcas() {
    nombre[0] = 'A'+f;
    nombre[1] = '1'+c;
    nombre[2] = 0;
    pardesnudo[enfila] = nullptr;
    pardesnudo[encolumna] = nullptr;
    pardesnudo[encaja] = nullptr;
}

void Casilla::Asignar(int v) {
    valor = v;
    marcas = MarcasLapiz();
    // Si la casilla tiene un valor final, ya no forma parte de un par desnudo
    if(pardesnudo[enfila]) MarcaParDesnudo(nullptr, enfila); // Destruimos el par desnudo
    if(pardesnudo[encolumna]) MarcaParDesnudo(nullptr, encolumna); // Destruimos el par desnudo
    if(pardesnudo[encaja]) MarcaParDesnudo(nullptr, encaja); // Destruimos el par desnudo
}

Casilla* Casilla::ParDesnudo(tipopardesnudo t) const {
    if(!pardesnudo[t]) return nullptr;
    return pardesnudo[t];
}

void Casilla::MarcaParDesnudo(Casilla* cas, tipopardesnudo t) {
    if(cas) {
        cas->pardesnudo[t] = this;
    } else {
        if(pardesnudo[t]) pardesnudo[t]->pardesnudo[t]=nullptr;
    }
    pardesnudo[t] = cas;
}

bool Casilla::ParDesnudo() {
    return ParDesnudo(enfila) || ParDesnudo(encolumna) || ParDesnudo(encaja);
}

void Casilla::DestruyeParDesnudo() {
    //if(ParDesnudo(enfila))
    MarcaParDesnudo(nullptr, enfila);
    //if(ParDesnudo(encolumna))
    MarcaParDesnudo(nullptr, encolumna);
    //if(ParDesnudo(encaja))
    MarcaParDesnudo(nullptr, encaja);
}
