// Sudokus con Clase
// Resolución de sudokus con técnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definición de la clase virtual Grupo

#include "grupo.h"

bool Grupo::Presente(int n) {
    for(int i=0; i<9; i++) if(cas[i]->Valor() == n) return true;
    return false;
}

void Grupo::Criba() {
    for(int i=0; i<9; i++) cas[i]->MarcaNoPosible();
}

void Grupo::CribaMenosCaja(int p) {
    for(int i=0; i<9; i++) if(i/3 != p) cas[i]->MarcaNoPosible();
}

Unico Grupo::CandidatoUnico() {
    bool valor[9] = {false, false, false, false, false, false, false, false, false};
    int cuenta = 0;
    int i, c;

    // Contar valores finales, si son 8 devolver valor que falta:
    for(i=0; i<9; i++) {
        if(cas[i]->Valor()) {
            cuenta++;
            valor[cas[i]->Valor()-1] = true;
        } else
            c=i;
    }
    if(cuenta == 8) {
        i=0;
        while(valor[i++]);
        return Unico(c, i);
    }
    return Unico(-1,0);
}

int Grupo::CuentaPosibles() {
    int cuenta=0;
    for(int i=0; i<9; i++)
        if(cas[i]->Posible()) cuenta++;
    return cuenta;
}

int Grupo::UnicoPosible() {
    int cuenta=0;
    int pos = -1;
    for(int i=0; i<9; i++) {
        if(cas[i]->Posible()) {
            cuenta++;
            pos = i;
        }
    }
    if(cuenta == 1) return pos;
    return -1;
}

int Grupo::CuentaFinales() {
    int cuenta=0;
    for(int i = 0; i<9; i++) {
        if(cas[i]->Valor()) cuenta++;
    }
    return cuenta;
}

int Grupo::PrimeroPosible() {
    for(int i=0; i<9; i++)
        if(cas[i]->Posible()) return i;
    return -1;
}

int Grupo::SegundoPosible(int p) {
    for(int i=p+1; i<9; i++)
        if(cas[i]->Posible()) return i;
    return -1;
}

int Grupo::TerceroPosible(int s) {
    for(int i=s+1; i<9; i++)
        if(cas[i]->Posible()) return i;
    return -1;
}

void Grupo::DesmarcaLapiz(int v) {
    for(int i=0; i<9; i++) cas[i]->Lapiz().EliminaMarca(v);
}

void Grupo::DesmarcaLapizMenosCaja(int k, int v) {
    for(int i=0; i<9; i++)
        if(i/3 != k) cas[i]->Lapiz().EliminaMarca(v);
}

Unico Grupo::UnicaCasillaConCandidato() {
    Unico u(-1, 0);
    int cuenta;

    for(int v=1; v<=9; v++) {
        cuenta = 0;
        for(int i=0; i<9; i++) {
            if(!cas[i]->Valor()) {
                if(cas[i]->Lapiz().ContieneMarca(v)) {
                    u = Unico(i, v);
                    cuenta++;
                }
            }
        }
        if(cuenta==1) return u;
    }
    return Unico(-1, 0);
}

Casilla *Grupo::BuscaAla1MenosCaja(Casilla *pivote, int i1) {
    int k = pivote->Caja();

    for(int i=i1; i<9; i++) {
        if(cas[i]->Caja() != k && cas[i]->Lapiz().CuentaMarcas() == 2) { // Distinta caja con dos marcas de lápiz
            if((cas[i]->Lapiz() | pivote->Lapiz()).CuentaMarcas() == 3) { // La casilla tiene un candidato en común con el pivote
                return cas[i];
            }
        }
    }
    return nullptr;
}

Casilla *Grupo::BuscaAla2MenosCaja(Casilla *pivote, Casilla *ala1) {
    int k = pivote->Caja();
    MarcasLapiz nocomunes = (pivote->Lapiz() | ala1->Lapiz()) - (pivote->Lapiz() & ala1->Lapiz());

    for(int i=0; i<9; i++) {
        if(cas[i]->Caja() != k && cas[i]->Lapiz() == nocomunes) { // Distinta caja con las dos marcas de lápiz comunes
            return cas[i];
        }
    }
    return nullptr;
}

Pareja Grupo::DobleCandidato(int v) {
    int cuenta=0;
    Pareja par;

    for(int i=0; i<9; i++) {
        if(cas[i]->Lapiz().ContieneMarca(v)) par.Asigna(cuenta++, cas[i]);
    }
    if(cuenta==2) return par;
    return Pareja();
}

int Grupo::CuentaCandidato(int v) {
    int cuenta = 0;

    for(int i=0; i<9; i++) {
        if(cas[i]->Lapiz().ContieneMarca(v)) cuenta++;
    }
    return cuenta;
}

int Grupo::BuscarMarcas(const MarcasLapiz &m) {

    for(int i=0; i<9; i++) {
        if(cas[i]->Lapiz() == m) return i;
    }
    return -1;
}

MarcasCasillas Grupo::Marcas(int v) {
    MarcasCasillas m;

    for(int i=0; i<9; i++) {
        if(cas[i]->Lapiz().ContieneMarca(v)) m.AsignaCasilla(i);
    }
    return m;
}

int Grupo::CasillasConCandidato(int v) {
    int cuenta=0;

    for(int i=0; i<9; i++) {
        if(cas[i]->Lapiz().ContieneMarca(v)) cuenta++;
    }
    return cuenta;
}
