// Sudokus con Clase
// Resolución de sudokus con técnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definición de la clase MarcasLapiz

#include <cstring>
#include "marcaslapiz.h"

bool MarcasLapiz::AsignaMarca(int v) {
    if(lapiz[v-1]) return false;
    lapiz[v-1] = true;
    return true;
}

bool MarcasLapiz::EliminaMarca(int v) {
    if(!lapiz[v-1]) return false;
    lapiz[v-1] = false;
    eliminada[v-1] = true;
    return true;
}

void MarcasLapiz::ResetEliminadas() {
    for(int v=0; v<9; v++) eliminada[v] = false;
}

int MarcasLapiz::CuentaMarcas() {
    int c=0;
    for(int v=0; v<9; v++) if(lapiz[v]) c++;
    return c;
}

MarcasLapiz MarcasLapiz::operator|(const MarcasLapiz &m1) {
    MarcasLapiz res;
    for(int v=0; v<9; v++) res.lapiz[v] = this->lapiz[v] || m1.lapiz[v];
    return res;
}

MarcasLapiz MarcasLapiz::operator&(const MarcasLapiz &m1) {
    MarcasLapiz res;
    for(int v=0; v<9; v++) res.lapiz[v] = this->lapiz[v] && m1.lapiz[v];
    return res;
}

MarcasLapiz MarcasLapiz::operator-(const MarcasLapiz &m1) {
    MarcasLapiz res;
    for(int v=0; v<9; v++)
        res.lapiz[v] = (m1.lapiz[v]) ? false : this->lapiz[v];
    return res;
}

bool MarcasLapiz::operator==(const MarcasLapiz &m1) {
    for(int v=0; v<9; v++) if(lapiz[v] != m1.lapiz[v]) return false;
    return true;
}

bool MarcasLapiz::operator!=(const MarcasLapiz &m1) {
    for(int v=0; v<9; v++) if(lapiz[v] != m1.lapiz[v]) return true;
    return false;
}

bool MarcasLapiz::Quitar(const MarcasLapiz &m1) {
    bool retval = false;
    for(int v=0; v<9; v++) {
        if(lapiz[v] && m1.lapiz[v]) {
            lapiz[v] = false;
            retval |= true;
        }
    }
    return retval;
}

char* MarcasLapiz::Valores() {
    static char cad[20];
    char val[2] = " ";

    std::strcpy(cad, "");
    for(int v=1; v<=9; v++) {
        if(ContieneMarca(v)) {
            val[0] = '0'+v;
            std::strcat(cad, val);
            std::strcat(cad, ",");
        }
    }
    cad[std::strlen(cad)-1]=0;
    return cad;
}

int MarcasLapiz::Valor() {
    if(CuentaMarcas() != 1) return 0;
    for(int v=0; v<9; v++) if(lapiz[v]) return v+1;
    return 0;
}

int MarcasLapiz::ValorMarca(int n) {
    for(int v=1; v<=9; v++) {
        if(ContieneMarca(v)) n--;
        if(n == 0) return v;
    }
    return -1;
}
