/***************************************************************
 * Name:      sudokuMain.cpp
 * Purpose:   Code for Application Frame
 * Author:    Salvador Pozo (salvador@conclase.net)
 * Created:   2025-01-11
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#include "sudokuMain.h"
#include "importarSudoku.h"

//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__WXMAC__)
        wxbuild << _T("-Mac");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-Unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
        wxbuild << _T("\nCon Clase");
        wxbuild << _T("\n2025 Salvador Pozo");
    }

    return wxbuild;
}

BEGIN_EVENT_TABLE(sudokuFrame, wxFrame)
    EVT_CLOSE(sudokuFrame::OnClose)
    EVT_PAINT(sudokuFrame::OnPaint)
    EVT_MENU(idMenuQuit, sudokuFrame::OnQuit)
    EVT_MENU(idMenuAbout, sudokuFrame::OnAbout)
    EVT_MENU(idMenuResolver, sudokuFrame::Resolver)
    EVT_MENU(idMenuAutomatico, sudokuFrame::Automatico)
    EVT_MENU(idMenuReiniciar, sudokuFrame::Reiniciar)
    EVT_MENU(idMenuLoad, sudokuFrame::Load)
    EVT_MENU(idMenuImportar, sudokuFrame::Importar)
END_EVENT_TABLE()

sudokuFrame::sudokuFrame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title, wxDefaultPosition, wxSize(800, 800)), ejecutandoHilo(false)
{
    SetBackgroundStyle(wxBG_STYLE_PAINT);
    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();
    wxMenu* fileMenu = new wxMenu(_T(""));
    fileMenu->Append(idMenuLoad, _T("&Cargar"), _T("Cargar juego"));
    fileMenu->Append(idMenuImportar, _T("&Importar"), _T("Importar juego"));
    fileMenu->Append(idMenuQuit, _T("&Salir\tAlt-F4"), _T("Abandonar la aplicación"));
    mbar->Append(fileMenu, _("&Fichero"));

    wxMenu* juegoMenu = new wxMenu(_T(""));
    juegoMenu->Append(idMenuResolver, _T("&Paso\tF2"), _T("Resolver paso a paso"));
    juegoMenu->Append(idMenuAutomatico, _T("&Resolver\tF3"), _T("Resolver juego"));
    juegoMenu->Append(idMenuReiniciar, _T("&Reiniciar\tF4"), _T("Reiniciar juego"));
    mbar->Append(juegoMenu, _("&Sudoku"));

    wxMenu* helpMenu = new wxMenu(_T(""));
    helpMenu->Append(idMenuAbout, _T("&Sobre\tF1"), _T("Muestra información sobre la aplicación"));
    mbar->Append(helpMenu, _T("&Ayuda"));

    SetMenuBar(mbar);

    wxBoxSizer *boxVertical;

    boxVertical = new wxBoxSizer(wxVERTICAL);
    boxVertical->AddSpacer(590);
    tecnica = new wxTextCtrl(this, idTextCtrl, wxEmptyString, wxDefaultPosition, wxSize(600, wxDefaultCoord), wxTE_READONLY);
    boxVertical->Add(tecnica);
    boxVertical->AddSpacer(10);
    log = new wxListBox(this, idListBox, wxDefaultPosition, wxSize(600,200));
    boxVertical->Add(log);
    SetSizerAndFit(boxVertical);

    CreateStatusBar(2);
    SetStatusText(_T("Resolución de Sudokus"),0);
    SetStatusText(wxbuildinfo(short_f), 1);

    T.Leer(tablero);
    Comprobar();
}


sudokuFrame::~sudokuFrame() {}

void sudokuFrame::OnClose(wxCloseEvent &event)
{
    Destroy();
}

void sudokuFrame::OnQuit(wxCommandEvent &event)
{
    Destroy();
}

void sudokuFrame::OnAbout(wxCommandEvent &event)
{
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _T("Resolución de sudokus..."));
}

void sudokuFrame::Load(wxCommandEvent& event) {
    wxString nombreFichero;
    wxFile* fichero;

    wxFileDialog LoadSudoku(this, _T("Carga sudoku"), wxEmptyString, wxEmptyString, "*.sud", wxFD_OPEN);
    if(LoadSudoku.ShowModal() == wxID_OK) {
        nombreFichero = LoadSudoku.GetPath();
        fichero = new wxFile(nombreFichero, wxFile::read);
        if(fichero->IsOpened()) {
            fichero->Read(static_cast<void*>(tablero), 82);
            T.Leer(tablero);
            tecnica->ChangeValue(_T(""));
            Comprobar();
            log->Clear();
            Refresh(true);
            SetStatusText(LoadSudoku.GetFilename(),0);
        }
        delete fichero;
    }
}

void sudokuFrame::Importar(wxCommandEvent& event) {
    importarSudoku *dlg = new importarSudoku(this, tablero);
    dlg->ShowModal();
    T.Leer(tablero);
    tecnica->ChangeValue(_T(""));
    Comprobar();
    log->Clear();
    Refresh(true);
    SetStatusText(_T("Importado"),0);
}

void sudokuFrame::Comprobar() {
    if(T.Cuenta() == -1) {
        T.SetCuenta(0);
        T.Soluciones();
        // Si la solución no es única, no intentar resolver
        if(T.Cuenta() != 1) {
            if(T.Cuenta() == 0)
                wxMessageBox(_T("El sudoku no tiene solución"), _T("Error"));
            else
                wxMessageBox(_T("El sudoku tiene varias soluciones"), _T("Error"));
            return;
        }
        T.SetEstado(inicio); // Cribas
    }

    if(T.Cuenta()!=1) return; // No se puede solucionar
}

void sudokuFrame::Reiniciar(wxCommandEvent& event) {
    T.Leer(tablero);
    tecnica->ChangeValue(_T(""));
    Comprobar();
    log->Clear();
    Refresh(true);
}

void sudokuFrame::Resolver(wxCommandEvent& event) {
    Paso();
    if(T.Estado() == resuelto)
        wxMessageBox(_T("Sudoku resuelto"), _T("Info"));
    if(T.Estado() == nosolucionable)
        wxMessageBox(_T("No se puede resolver"), _T("Info"));
}

void sudokuFrame::Automatico(wxCommandEvent& event) {
    while(T.Estado() != resuelto && T.Estado() != nosolucionable) Paso();
    if(T.Terminado())
        wxMessageBox(_T("Sudoku resuelto"), _T("Info"));
    else
        wxMessageBox(_T("No se puede resolver"), _T("Info"));
}

void sudokuFrame::Paso() {
    wxString cad;

    if(T.Cuenta()!=1) return; // No se puede solucionar (Desactivar menús)
    if(T.Terminado()) T.SetEstado(resuelto);

    T.LimpiaResaltes();
    switch(T.Estado()) {
    case inicio:
        valor=1; // Empezar las cribas con 1
    case cribado:
        while(valor<10 && !T.Criba(valor)) valor++;
        if(valor<10) {
            cad = wxString::Format(_T("Cribado con valor %d"), valor);
            tecnica->ChangeValue(cad);
            Log();
            Refresh(true);
            break;
        }
    case paresdesnudos:
        T.BorraCriba();
        T.BuscaParesDesnudos(false); // Actualiza la lista de pares desnudos
        if(T.ProcesarParesDesnudos(false)) {
            cad = wxString::Format(_T("Buscar y procesar pares desnudos"));
            tecnica->ChangeValue(cad);
            Log();
            Refresh(true);
            break;
        }
    case marcaslapiz:
        T.BorraCriba();
        cad = wxString(_T("Añadir marcas de lápiz"));
        tecnica->ChangeValue(cad);
        T.PonerMarcasLapiz();
        Log();
        T.SetEstado(unamarcalapiz);
        Refresh(true);
        break;
    case unamarcalapiz:
        if(T.CasillasConUnaMarcaLapiz()) {
            cad = wxString(_T("Buscar casillas con sólo una marca de lápiz"));
            tecnica->ChangeValue(cad);
            Log();
            Refresh(true);
            break;
        }
    case candidatounicofila:
        if(T.UnicaCasillaConCandidato()) {
            cad = wxString(_T("Buscar casilla con el único candidato para un valor en filas, columnas y cajas"));
            tecnica->ChangeValue(cad);
            Log();
            Refresh(true);
            break;
        }
    case buscaparesdesnudos2:
        if(T.BuscaParesDesnudos()) { // Actualiza la lista de pares desnudos
            cad = wxString(_T("Procesar pares desnudos"));
            tecnica->ChangeValue(cad);
            Log();
            Refresh(true);
            break;
        }
    case procesaparesdesnudos2:
        if(T.ProcesarParesDesnudos()) {
            cad = wxString(_T("Procesar pares desnudos"));
            tecnica->ChangeValue(cad);
            Log();
            Refresh(true);
            break;
        }
    case punteros:
        T.BuscaPunteros();
        if(T.ProcesaPunteros()) {
            cad = wxString(_T("Busca y procesa pares y trios punteros"));
            tecnica->ChangeValue(cad);
            Log();
            Refresh(true);
            break;
        }
    case grupos2:
        if(T.ProcesaGrupo(2)) {
            tecnica->ChangeValue("Grupos de dos");
            Log();
            Refresh(true);
            break;
        }
    case grupos3:
        if(T.ProcesaGrupo(3)) {
            tecnica->ChangeValue("Grupos de tres");
            Log();
            Refresh(true);
            break;
        }
    case grupos4:  // Faltan los grupos de 5, 6 y 7
        if(T.ProcesaGrupo(4)) {
            tecnica->ChangeValue("Grupos de cuatro");
            Log();
            Refresh(true);
            break;
        }
    case grupos5:  // Faltan los grupos de 5, 6 y 7
        if(T.ProcesaGrupo(5)) {
            tecnica->ChangeValue("Grupos de cinco");
            Log();
            Refresh(true);
            break;
        }
    case xywing:
        if(T.ProcesaXYWing()) {
            tecnica->ChangeValue("Procesar XY-Wing");
            Log();
            Refresh(true);
            break;
        }
    case xwing:
        if(T.ProcesaXWing()) {
            tecnica->ChangeValue("Procesar X-Wing");
            Log();
            Refresh(true);
            break;
        }
    case doscandidatos:
        if(T.TodasConDosCandidatosMenosUna()) {
            tecnica->ChangeValue("Todas las casillas con dos candidatos menos una");
            Log();
            Refresh(true);
            break;
        }
    case pezespada:
        if(T.ProcesaSwordFish()) {
            tecnica->ChangeValue("Procesar pez espada");
            Log();
            Refresh(true);
            break;
        }
    case xwingalas:
        if(T.ProcesaXWingAlas()) {
            tecnica->ChangeValue("Procesar X-Wing Alas");
            Log();
            Refresh(true);
            break;
        }
    case solucionunicaA:
        if(T.SolucionUnicaA()) {
            tecnica->ChangeValue("Solucion unica tipo A");
            Log();
            Refresh(true);
            break;
        }
    case solucionunicaB:
        if(T.SolucionUnicaB()) {
            tecnica->ChangeValue("Solucion unica tipo B");
            Log();
            Refresh(true);
            break;
        }
    case stringkite:
        if(T.StringKite()) {
            tecnica->ChangeValue("Procesar hilo de cometa (StringKite)");
            Log();
            Refresh(true);
            break;
        }
    case xyzwing:
        if(T.ProcesaXYZWing()) {
            tecnica->ChangeValue("Procesa XYZ-Wing");
            Log();
            Refresh(true);
            break;
        }
    case sashimi:
        if(T.ProcesaSashimi()) {
            tecnica->ChangeValue("Procesa Sashimi X-Wing");
            Log();
            Refresh(true);
            break;
        }
    case skyscraper:
        if(T.ProcesaSkyscraper()) {
            tecnica->ChangeValue("Procesa Skyscraper");
            Log();
            Refresh(true);
            break;
        }
    case rectangulovacio:
        if(T.ProcesaRectangulosVacios()) {
            tecnica->ChangeValue("Procesa rectangulos vacios");
            Log();
            Refresh(true);
            break;
        }
    case medusa:
        if(T.ProcesaJellyFish()) {
            tecnica->ChangeValue("Procesa Medusa");
            Log();
            Refresh(true);
            break;
        }
    case pezespadaalas:
        if(T.SwordFishAlas()) {
            tecnica->ChangeValue("Procesa pez espada con alas");
            Log();
            Refresh(true);
            break;
        } else {
            T.SetEstado(nosolucionable);
        }
    case nosolucionable:
        T.BorraCriba();
        break;
    case resuelto:
        T.BorraCriba();
        Refresh(true);
    }
}

void sudokuFrame::Log() {
    std::string cad;
    while(T.Mensaje(cad))
        log->EnsureVisible(log->Append(cad));
}

void sudokuFrame::OnPaint(wxPaintEvent& event) {
    int w, h;
    GetClientSize(&w, &h);
    if(w==0) return;
    wxBufferedPaintDC dc(this);

    dc.Clear();
    int lado = 60;
    wxFont fuenteNumero(2*lado/4, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_ITALIC, wxFONTWEIGHT_NORMAL);
    wxString cad;
    wxPen plumaGris(wxColour(40,40,40));
    wxBrush pincelVerde(wxColour(120,250,120));
    wxBrush pincelAzul(wxColour(100,100,250));
    wxBrush pincelBlanco(wxColour(255,255,255));
    wxBrush pincelGris(wxColour(220,220,220));
    wxBrush pincelAmarillo(wxColour(220,220,100));
    wxBrush pincelNaranja(wxColour(255,180,130));
    int dx=0;
    int dy=0;

    wxRect re(wxPoint(0,0), wxSize(lado,lado));

    dc.SetPen(plumaGris);
    dc.DrawLine(wxPoint(40+lado*3,40), wxPoint(40+lado*3,42+lado*9));
    dc.DrawLine(wxPoint(41+lado*6,40), wxPoint(41+lado*6,42+lado*9));
    dc.DrawLine(wxPoint(40,40+lado*3), wxPoint(42+lado*9,40+lado*3));
    dc.DrawLine(wxPoint(40,41+lado*6), wxPoint(42+lado*9,41+lado*6));

    for(int f=0; f<9; f++) {
        dc.SetFont(fuenteNumero);
        dc.SetTextForeground(wxColour(40,40,40));
        cad = wxString::Format("%c", 'A'+f);
        dc.DrawText(cad, wxPoint(5,42+lado*f));
        cad = wxString::Format("%c", '1'+f);
        dc.DrawText(cad, wxPoint(50+lado*f,1));
        for(int c=0; c<9; c++) {
            dx=40;
            if(c>2) dx+=1;
            if(c>5) dx+=1;
            dy=40;
            if(f>2) dy+=1;
            if(f>5) dy+=1;
            re.SetPosition(wxPoint(dx+c*lado, dy+f*lado));
            if(T.GetCasilla(f,c).EsAmarillo()) dc.SetBrush(pincelAmarillo);
            else if(T.GetCasilla(f,c).EsNaranja()) dc.SetBrush(pincelNaranja);
            else if(T.GetCasilla(f,c).Resaltado()) dc.SetBrush(pincelAzul);
            else if(T.GetCasilla(f,c).Inicial()) dc.SetBrush(pincelGris);
            else if(T.GetCasilla(f,c).Posible()) dc.SetBrush(pincelVerde);
            else dc.SetBrush(pincelBlanco);
            dc.DrawRectangle(re);
            if(T.GetCasilla(f,c).Valor())
                MuestraValor(dc, lado, T.GetCasilla(f,c));
            else
                MuestraMarcasDeLapiz(dc, lado, T.GetCasilla(f,c));
        }
    }
}

void sudokuFrame::MuestraValor(wxDC& dc, int lado, Casilla& cas) {
    wxFont fuenteNumero(3*lado/4, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
    wxString cad;
    int x, y;

    dc.SetFont(fuenteNumero);
    x = cas.Columna();
    y = cas.Fila();
    wxColour col;
    if(cas.Inicial()) col = wxColour(0,0,0);
    else col = wxColour(80,80,200);
    dc.SetTextForeground(col);
    int tx=35+x/3+2*lado/7;
    int ty=39;
    cad = wxString::Format("%1d", cas.Valor());
    dc.DrawText(cad, wxPoint(tx+lado*x,ty+lado*y));

}

void sudokuFrame::MuestraMarcasDeLapiz(wxDC& dc, int lado, Casilla &cas) {
    wxFont fuenteLapiz(lado/3-4, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
    int x, y;
    int tx, ty;
    wxString cad;

    x = cas.Columna();
    y = cas.Fila();
    tx=35+x/3+2*lado/7;
    ty=39;
    dc.SetFont(fuenteLapiz);
    for(int v=1; v<=9; v++) {
        if(cas.Lapiz().ContieneMarca(v) || cas.Lapiz().ContieneEliminada(v)) {
            cad = wxString::Format("%1d", v);
            tx=41+x*lado;
            ty=39+y*lado;
            switch((v-1)%3) {
                case 2: tx+=lado/3;
                case 1: tx+=lado/3;
            }
            switch((v-1)/3) {
                case 2: ty+=lado/3;
                case 1: ty+=lado/3;
            }
            if(cas.Lapiz().ContieneMarca(v)) {
                dc.SetTextForeground(wxColour(150,150,150));
            }
            if(cas.Lapiz().ContieneEliminada(v)) {
                dc.SetTextForeground(wxColour(250,30,30));
            }
            dc.DrawText(cad, wxPoint(tx,ty));
        }
    }
}
